@@ -1,4 +1,3 @@
-import base64
 from typing import Optional, List, Dict, Any, TYPE_CHECKING, Union
 
 from pydantic import BaseModel, validator
@@ -6,13 +5,14 @@ from pydantic import BaseModel, validator
 from ..math.ndarray import to_list
 
 if TYPE_CHECKING:
-    from ..types import ArrayType
+    from ..typing import ArrayType
 
 # this order must be preserved: https://pydantic-docs.helpmanual.io/usage/types/#unions
-_ProtoValueType = Optional[Union[bool, float, str]]
+_ProtoValueType = Optional[Union[bool, float, str, list, dict]]
 _StructValueType = Union[
     _ProtoValueType, List[_ProtoValueType], Dict[str, _ProtoValueType]
 ]
+_MetadataType = Dict[str, _StructValueType]
 
 
 def _convert_ndarray_to_list(v: 'ArrayType'):
@@ -27,18 +27,23 @@ class _NamedScore(BaseModel):
     ref_id: Optional[str] = None
 
 
+class _MetadataModel(BaseModel):
+    metadata: _MetadataType
+
+
 class PydanticDocument(BaseModel):
-    id: str
+    id: Optional[str]
     parent_id: Optional[str]
     granularity: Optional[int]
     adjacency: Optional[int]
-    blob: Optional[bytes]
+    blob: Optional[str]
     tensor: Optional[Any]
     mime_type: Optional[str]
     text: Optional[str]
     weight: Optional[float]
     uri: Optional[str]
     tags: Optional[Dict[str, '_StructValueType']]
+    _metadata: Optional[Dict[str, '_StructValueType']]
     offset: Optional[float]
     location: Optional[List[float]]
     embedding: Optional[Any]
@@ -51,17 +56,17 @@ class PydanticDocument(BaseModel):
     _tensor2list = validator('tensor', allow_reuse=True)(_convert_ndarray_to_list)
     _embedding2list = validator('embedding', allow_reuse=True)(_convert_ndarray_to_list)
 
-    @validator('blob')
-    def _blob2base64(cls, v):
-        if v is not None:
-            if isinstance(v, bytes):
-                return base64.b64encode(v).decode('utf8')
-            else:
-                raise ValueError('must be bytes')
-
     class Config:
         smart_union = True
 
+    def __init__(self, **data):
+        super().__init__(**data)
+        # underscore attributes need to be set and validated manually
+        _metadata = data.get('_metadata', None)
+        if _metadata is not None:
+            _md_model = _MetadataModel(metadata=_metadata)  # validate _metadata
+            object.__setattr__(self, '_metadata', _md_model.metadata)
+
 
 PydanticDocument.update_forward_refs()
 
